/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource;

import inti.util.DateFormatter;
import inti.ws.spring.resource.minify.Minifier;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.servlet.ServletContext;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.io.IOUtils;
import org.springframework.web.context.support.ServletContextResource;

public class WebResource {
    
    protected static final Charset UTF8 = Charset.forName("utf-8");
    protected static final DateFormatter DATE_FORMATTER = new DateFormatter();
    
    protected boolean minify;
    protected long lastModified = 0;
    protected Minifier minifier;
    protected String compressedFile;
    protected String messageDigest;
    protected String lastModifiedString;
    protected ServletContextResource resource;
    protected ThreadLocal<MessageDigest> digests = new ThreadLocal<MessageDigest>() {
        
        @Override
        protected MessageDigest initialValue() {
            try {
                return MessageDigest.getInstance("MD5");
            } catch (NoSuchAlgorithmException exception) {
                return null;
            }
        }
        
    };
    
    public WebResource(ServletContext ctx, String location, Minifier minifier, boolean minify)
            throws URISyntaxException, MalformedURLException {
        this.minifier = minifier;
        this.minify = minify;
        if (location != null) {
            resource = new ServletContextResource(ctx, location);
        }
    }
    
    public String getName() {
        return resource.getFilename();
    }
    
    public boolean hasChanged() throws IOException {
        return resource.lastModified() > lastModified;
    }
    
    public boolean updateIfNeeded() throws IOException, Exception {
        if (hasChanged()) {
            update();
            return true;
        }
        return false;
    }
    
    public void update() throws Exception {
        StringBuilder builder = new StringBuilder(32);
        MessageDigest digest = digests.get();
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        InputStream inputStream = resource.getInputStream();
        
        try {
            lastModified = resource.lastModified();
            IOUtils.copyLarge(inputStream, outputStream);
            compressedFile = new String(outputStream.toByteArray(), "utf-8");
        } finally {
            inputStream.close();
        }
        
        if (minify) {
            compressedFile = minifier.minify(compressedFile);
        }
        
        digest.reset();
        builder.append(Hex.encodeHexString(digest.digest(compressedFile.getBytes(UTF8))));
        messageDigest = builder.toString();
        builder.delete(0, builder.length());
        
        DATE_FORMATTER.formatDate(lastModified, builder);
        lastModifiedString = builder.toString();
    }
    
    public long getLastModified() {
        return lastModified;
    }
    
    public String getLastModifiedString() {
        return lastModifiedString;
    }
    
    public String getMessageDigest() {
        return messageDigest;
    }
    
    public String getContent() {
        return compressedFile;
    }
    
}
